@@ -535,6 +535,30 @@ class Translation:
     value: str
     comment: str
 
+class FunctionFile(FileResource):
+    """
+    A FunctionFile is a function file, such as run.mcfunction, and is
+    made up of many commands.
+    """
+    def __init__(self, file_path: str = None, pack: Pack = None) -> None:
+        super().__init__(file_path=file_path, pack=pack)
+        self.__commands : list[str] = []
+    
+    @cached_property
+    def commands(self) -> list[str]:
+        with open(os.path.join(self.pack.input_path, self.file_path), "r", encoding='utf-8') as function_file:
+            for line in function_file.readlines():
+                if line.strip() and not line.startswith("#"):
+                    self.__commands.append(line)
+        return self.__commands
+
+    def _save(self):
+        path = os.path.join(self.pack.output_path, self.file_path)
+        create_nested_directory(path)
+        with open(path, 'w', encoding='utf-8') as file:
+            for command in self.commands:
+                file.write(command)
+
 class LanguageFile(FileResource):
     """
     A LanguageFile is a language file, such as 'en_US.lang', and is made
@@ -660,6 +684,12 @@ class Pack():
         Register a child resource to this pack. These resources will be saved
         during save.
         """
+
+        # Attempt to set the pack for this resource.
+        try:
+            resource.pack = self
+        except Exception:
+            pass
         self.resources.append(resource)
 
     def get_language_file(self, file_name:str) -> LanguageFile:
@@ -734,7 +764,7 @@ class Project():
         self.__behavior_path = behavior_path
         self.__resource_path = resource_path
         self.__resource_pack = None
-        self.__behavior_pack = None
+        self.__behavior_pack : BehaviorPack = None
 
     @cached_property
     def resource_pack(self) -> ResourcePack:
@@ -941,6 +971,7 @@ class ResourcePack(Pack):
 class BehaviorPack(Pack):
     def __init__(self, input_path: str, project: Project = None):
         super().__init__(input_path, project=project)
+        self.__functions = []
         self.__features_file = []
         self.__feature_rules_files = []
         self.__spawn_rules = []
@@ -952,6 +983,15 @@ class BehaviorPack(Pack):
         self.__blocks = []
         
     
+    @cached_property
+    def functions(self) -> list[FunctionFile]:
+        base_directory = os.path.join(self.input_path, "functions")
+        for local_path in glob.glob(base_directory + "/**/*.mcfunction", recursive=True):
+            local_path = os.path.relpath(local_path, self.input_path)
+            self.__functions.append(FunctionFile(file_path = local_path, pack = self))
+            
+        return self.__functions
+
     @cached_property
     def features_file(self) -> list[FeaturesFileBP]:
         base_directory = os.path.join(self.input_path, "features")
@@ -1035,6 +1075,12 @@ class BehaviorPack(Pack):
 
     
     
+    def get_function(self, file_path:str) -> FunctionFile:
+        for child in self.functions:
+            if child.file_path == file_path:
+                return child
+        raise AssetNotFoundError(file_path)
+
     def get_feature_rules_file(self, identifier:str) -> FeatureRulesFileBP:
         for child in self.feature_rules_files:
             if child.identifier == identifier:
